<?php

namespace App\Http\Controllers;

use App\Http\Requests\GetOTPRequest;
use App\Http\Requests\LoginRequest;
use App\Http\Requests\RegisterRequest;
use App\Http\Requests\User\ChangePassword;
use App\Http\Requests\User\UpdateUser;
use App\Http\Requests\ValidateOTPRequest;
use App\Repository\Eloquent\UserRepository;
use Illuminate\Http\Request;
use Symfony\Component\HttpKernel\EventListener\ValidateRequestListener;

class UserController extends Controller
{

    protected $userRepository;

    public function __construct(UserRepository $userRepository)
    {
        $this->userRepository = $userRepository;
    }

    public function register(RegisterRequest $request)
    {
        return $this->userRepository->register($request);
    }

    public function login(LoginRequest $request)
    {
        // return response()->json($this->userRepository->loginWithEmail($request), 200);
        return $this->userRepository->login($request);
    }

    public function logout()
    {
        return $this->userRepository->logout();
    }

    public function getOTPCode(GetOTPRequest $request)
    {
        return $this->userRepository->getOTPCode($request);
    }

    public function validateOTPCode(ValidateOTPRequest $request)
    {
        return $this->userRepository->validateOTPCode($request);
    }

    public function favorites()
    {
        return $this->userRepository->favorites();
    }

    public function addToFavorites(Request $request)
    {
        return $this->userRepository->addToFavorites($request);
    }

    public function deleteFavorites(Request $request)
    {
        return $this->userRepository->deleteFavorites($request);
    }

    public function itemLiked()
    {
        return $this->userRepository->itemLiked();
    }

    public function info()
    {
        return $this->userRepository->info();
    }

    public function updateUserInfo(UpdateUser $request)
    {
        return $this->userRepository->updateUser($request);
    }

    public function changePassword(ChangePassword $request)
    {
        return $this->userRepository->changePassword($request);
    }

    public function resetPassword(Request $request)
    {
        return $this->userRepository->resetPassword($request);
    }

    // public function plans()
    // {
    //     return $this->userRepository->plans();
    // }
}
