<?php

namespace App\Http\Services;

use App\Http\Requests\GetOTPRequest;
use App\Http\Requests\LoginRequest;
use App\Http\Requests\RegisterRequest;
use App\Http\Requests\ValidateOTPRequest;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Hash;

class UserServices
{
    
    public function saveUser(RegisterRequest $userData)
    {
        $userData = $userData->toArray();
        $userData['password'] = Hash::make($userData['password']);
        $user = User::create($userData);
        $data['token'] =  $user->createToken('token')->plainTextToken;
        $data['user'] =  User::find($user->id);

        return $data;
    }

    public function checkLoginInfo(LoginRequest $request, $with = 'email')
    {

        $data = [];
        $user = null;
        if($with === 'email')
        {
            if(Auth::attempt(['email' => $request->email, 'password' => $request->password])){ 
                $user = Auth::user();
                $data['token'] = $user->createToken('token')->plainTextToken;
    
            } 
        } elseif($with === 'phone_number') {
            
            if(Auth::attempt(['phone_number' => $request->phone_number, 'password' => $request->password])){ 
                $user = Auth::user();
                $data['token'] =  $user->createToken('token')->plainTextToken;
                $data['user'] =  $user;
            }

        }

        if(!is_null($user)){

            $subsubscription = [];
            unset($user['subscription']);
            if(!is_null($user->subscription_end)){
                $subsubscription['has'] = now()->lt($user->subscription_end);
            } else {
                $subsubscription['has'] = false;
            }

            $subsubscription['days']   = now()->diffInDays($user->subscription_end) + 1;
            $subsubscription['start']  = $user->subscription_start;
            $subsubscription['end']    = $user->subscription_end;
            $user['subscription'] = $subsubscription;

            $data['user'] =  $user;
        }


        return $data;
    }

    public function createOTPCode(GetOTPRequest $request)
    {

        return Cache::remember('OTP' . $request->phone_number, now()->addSeconds(120), function() {
            return rand(100000, 999999);
        });
        
    }

    public function validateOTPCode(ValidateOTPRequest $request)
    {

        $otpCode = Cache::get('OTP' . $request->phone_number);

            if ($otpCode == $request->otp_code) {
                Cache::forget('OTP' . $request->phone_number);
                return true;
            } else {
                return false;
            }
        
    }

}
