<?php

namespace App\Models;

// use Illuminate\Contracts\Auth\MustVerifyEmail;

use Illuminate\Contracts\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Sanctum\HasApiTokens;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'first_name',
        'last_name',
        'postal_code',
        'address',
        'email',
        'subscription',
        'score',
        'phone_number',
        'password',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    public function scopeLatest(Builder $builder)
    {
        return $builder->orderBy(static::CREATED_AT, 'desc');
    }

    public function comments()
    {
        return $this->hasMany(Comment::class);
    }

    public function itemLikes()
    {
        return $this->morphedByMany(Item::class, 'like');
    }

    public function commentLikes()
    {
        return $this->morphedByMany(Comment::class, 'like');
    }

    public function answersLikes()
    {
        return $this->morphedByMany(Answer::class, 'like');
    }

    public function transactions()
    {
        return $this->hasMany(Transaction::class);
    }

    public function plans()
    {
        return $this->hasMany(Plan::class)->latest();
    }

    public function tickets()
    {
        return $this->hasMany(Ticket::class);
    }

    public function usedDiscountCodes()
    {
        return $this->belongsToMany(DiscountCode::class, 'used_discount_code');
    }

    public function favorites()
    {
        return $this->belongsToMany(Item::class, 'favorites');
    }

    public function answers()
    {
        return $this->hasMany(Answer::class);
    }
}
