<?php

namespace App\Providers;

use Illuminate\Cache\RateLimiting\Limit;
use Illuminate\Foundation\Support\Providers\RouteServiceProvider as ServiceProvider;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\Facades\Route;

class RouteServiceProvider extends ServiceProvider
{
    /**
     * The path to the "home" route for your application.
     *
     * Typically, users are redirected here after authentication.
     *
     * @var string
     */
    public const HOME = '/';

    /**
     * Define your route model bindings, pattern filters, and other route configuration.
     *
     * @return void
     */
    public function boot()
    {
        $this->configureRateLimiting();

        $this->routes(function () {
            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/api.php'));

            Route::middleware('api')
                ->prefix('api/user')
                ->group(base_path('routes/user.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/item.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/category.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/ticket.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/discountCode.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/plan.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/questionPackage.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/question.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/answer.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/like.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/setting.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/notification.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/home.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/search.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/comment.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/email.php'));

            Route::middleware('api')
                ->prefix('api')
                ->group(base_path('routes/film_cast.php'));

            Route::middleware('web')
                ->group(base_path('routes/web.php'));
        });
    }

    /**
     * Configure the rate limiters for the application.
     *
     * @return void
     */
    protected function configureRateLimiting()
    {
        RateLimiter::for('api', function (Request $request) {
            return Limit::perMinute(60)->by($request->user()?->id ?: $request->ip());
        });
    }
}
